/*
 * SDL port done by exelenz (ivan@solvo.ru) @ 14/10/2001.
 */


/****************************************************************************\
*  *
* File 'egl.cpp'  - EXeLENZ Graphics Library $Revision: 6 $                  *
* Project started - 22:34:39, 20.09.97                                       *
* Last modified   - $Date: 14.03.99 12:21 $                                  *
*                                                                            *
* Written by EXeLENZ (2:5030/448.1@fidonet, ivan@solvo.ru)                   *
* Changes and additions by Yoletir (55:1055/246@general, 2:5030/654@fidonet) *
*  *
*/

#include <malloc.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include "egl_sdl.h"
#include "egl_serv.h"
#include "egl_defs.h"

#include "SDL.h"   /* All SDL App's need this */
SDL_Surface *SDLscreen;

VBuffer EGLvBuffer;             // Main video surface for egl

static int XResolution, YResolution, IsFullscreen, WndXStart, WndYStart, BufX, BufY;
int NoStretch=0;

char EGLbytesPerPixel;

unsigned short *ScaleBuf = NULL;

// ? what is real difference between hw and sw surfaces? speed/support
#define SURFACEOPTS SDL_HWSURFACE|SDL_DOUBLEBUF

////
// initialisation/service routines                                          //
////


struct EGLmodeEnum {
   unsigned int x, y, bpp;
   int available;
};

/****************************************************************************\
* Function : EGLhaveMode                                                     *
* Desc     : Determines if particular video mode is supported                *
* Input    : resolution, bpp                                                 *
* Output   : success/failure                                                 *
****************************************************************************/
int EGLhaveMode (int x, int y, int bpp) {

   return SDL_VideoModeOK(x, y, bpp, SURFACEOPTS);

}

/****************************************************************************\
* Function : EGLinitSDL                                                      *
* Desc     : Sets needed video mode                                          *
* Input    : x,y, bpp, if it is fullscreen. rest is obsolete                 *
* Output   : success/failure                                                 *
\****************************************************************************/
int EGLinitSDL (int x, int y, int bpp, int bFullscreen,
                int BufX, int BufY, int bNoStretch) {


   /* Initialize defaults, Video */
   if((SDL_Init(SDL_INIT_VIDEO)==-1)) {
      printf("Could not initialize SDL: %s.\n", SDL_GetError());
      exit(-1);
   }

   if (bFullscreen==1) {
      SDLscreen = SDL_SetVideoMode(x, y, bpp, SURFACEOPTS|SDL_FULLSCREEN);
   } else {
      SDLscreen = SDL_SetVideoMode(x, y, bpp, SURFACEOPTS);
   }

   if ( SDLscreen == NULL ) {
      fprintf(stderr, "Couldn't set %dx%dx%d video mode: %s\n",
                       x,y,bpp,SDL_GetError());
      exit(1);
   }

   SDL_WM_SetCaption ("Electron Soccer (SDL)","");

   switch (bpp) {
      case 4:  EGLbytesPerPixel=1;break;
      case 8:  EGLbytesPerPixel=1;break;
      case 15: EGLbytesPerPixel=2;break;
      case 16: EGLbytesPerPixel=2;break;
      case 24: EGLbytesPerPixel=4;break;
      case 32: EGLbytesPerPixel=4;break;
      default: EGLbytesPerPixel=1;
   }


   BufX=320;BufY=200;

   XResolution = x; YResolution = y; IsFullscreen = bFullscreen;
   ::BufX = BufX; ::BufY = BufY;
   NoStretch = bNoStretch;

   return EGLok;
}

/****************************************************************************\
* Function : EGLterminate                                                    *
* Desc     : Terminate EGL                                                   *
* Input    : none                                                            *
* Output   : none                                                            *
\****************************************************************************/
void EGLterminate (void) {

   if (ScaleBuf != NULL) free (ScaleBuf);
   SDL_Quit ();
}

////
// Virtual buffer                                                           //
////

/****************************************************************************\
* Function : EGLcreateVBuffer                                                *
* Desc     : Creates virtual bufer                                           *
* Input    : pointer to v buffer                                             *
* Output   : success/failure                                                 *
\****************************************************************************/
int EGLcreateVBuffer (VBuffer * buffer)
{
   if (buffer == NULL) return EGLerror;

   // Allocate memory 4 buffer
   if ((*buffer=(VBuffer) malloc
      (BufX*BufY*EGLbytesPerPixel))==NULL)
      return (EGLerror);

   // Set to zeroes
   memset (*buffer,0,BufX*BufY*EGLbytesPerPixel);

   return (EGLok);
}

/****************************************************************************\
* Function : EGLsetActiveVBuffer                                             *
* Desc     : Set active virtual buffer                                       *
* Input    : buffer                                                          *
* Output   : success/failure                                                 *
\****************************************************************************/
int EGLsetActiveVBuffer (VBuffer buffer)
{
   if (buffer == NULL) return EGLerror;

   // Set pointer to active buffer
   EGLvBuffer=buffer;

   return EGLok;
}

/****************************************************************************\
* Function : EGLclearVBuffer                                                 *
* Desc     : Clear buffer                                                    *
* Input    : buffer                                                          *
* Output   : success/failure                                                 *
\****************************************************************************/
int EGLclearVBuffer (VBuffer buffer) {
   if (buffer == NULL) return EGLerror;

   memset (buffer,0,BufX*BufY*EGLbytesPerPixel);
   return EGLok;
}

/****************************************************************************\
* Function : EGLdrawVBuffer                                                  *
* Desc     : Draw buffer on screen                                           *
* Input    : virtual buffer                                                  *
* Output   : ok/error                                                        *
\****************************************************************************/
int EGLdrawVBuffer (VBuffer buffer) {

   if (buffer == NULL) return EGLerror;

   SDL_LockSurface (SDLscreen);
   SDLscreen->pixels=buffer;
   SDL_UnlockSurface (SDLscreen);

   SDL_Flip(SDLscreen);

   return EGLok;
}

////
// Sprite handling                                                          //
////

/****************************************************************************\
* Function : EGLallocBlock                                                   *
* Desc     : Alllocate memory for eglBlock                                   *
* Input    : BitMap, X size of image, Y size of image                        *
* Output   : EGLok/EGLerror                                                  *
\****************************************************************************/
int EGLallocBlock (BitMap ** block,
                   short xsize,
                   short ysize)
{
   if (block == NULL) return EGLerror;

   // Allocate needed mem block
   (*block)=(struct BitMap *) malloc (sizeof (BitMap));

   uchar *temp=(uchar *) malloc ((int)xsize*(int)(ysize+2)*EGLbytesPerPixel);
   if (temp==NULL) return (EGLerror);

   // Set block' to this chunk of memory
   (*block)->Image=temp;

   // Set sprite size
   (*block)->XSize=xsize;
   (*block)->YSize=ysize;

   return (EGLok);
}

/****************************************************************************\
* Function : EGLfreeBlock                                                    *
* Desc     : Free sprite                                                     *
* Input    : BitMap                                                          *
* Output   : EGLok/EGLerror                                                  *
\****************************************************************************/
int EGLfreeBlock (BitMap * block)
{
   if (block == NULL) return EGLerror;

   if (block->Image != NULL) free (block->Image);
   free (block);
   return EGLok;
}

/****************************************************************************\
* Function : EGLgetBlock                                                     *
* Desc     : Stores rectangle region of VBuffer in BitMap                    *
* Input    : Destination bitmap, x ,y ,xrl ,yrl                              *
* Output   : EGLok/EGLerror                                                  *
\****************************************************************************/
int EGLgetBlock (BitMap* block,
                  ushort x,
                  ushort y,
                  ushort xrl,
                  ushort yrl)
{

   if (block == NULL) return EGLerror;

   block->XSize=xrl-x+1;                  /* Adjust block X size            */
   block->YSize=yrl-y+1;                  /* Adjust block Y size            */

   int xpos_start=x*EGLbytesPerPixel, xpos_end=xrl*EGLbytesPerPixel;
   for (ushort yc=y; yc<=yrl; yc++) {
      memcpy (block->Image + ((yc-y)*block->XSize)*EGLbytesPerPixel,
              EGLvBuffer + (yc*BufX+x)*EGLbytesPerPixel,
              (xrl-x+1)*EGLbytesPerPixel);
   }

   return EGLok;
}

/****************************************************************************\
* Function : EGLputBlock                                                     *
* Desc     : Puts rectangle region of picture in EGLvBuffer                  *
* Input    : Source bitmap, x ,y                                             *
* Output   : EGLok/EGLerror                                                  *
\****************************************************************************/
int EGLputBlock (BitMap* block,
                  ushort x,
                  ushort y) {

   if (block == NULL) return EGLerror;

   ushort xrl,yrl;

   if (y+block->YSize > BufY) yrl=BufY-y;
   else yrl = block->YSize;
   if (x+block->XSize > BufX) xrl=BufX-x;
   else xrl = block->XSize;

   SDL_LockSurface (SDLscreen);

   for (ushort yc=0; yc<yrl; yc++) {
      memcpy (EGLvBuffer + ((yc+y)*BufX+x)*EGLbytesPerPixel,
              block->Image + (yc*block->XSize)*EGLbytesPerPixel,
              xrl*EGLbytesPerPixel);
   }

   SDL_UnlockSurface (SDLscreen);

   return EGLok;
}

/****************************************************************************\
* Function : EGLwaitVRetrace                                                 *
* Desc     : Wait for vertical retrace                                           *
* Input    : None                                                            *
* Output   : None                                                            *
\****************************************************************************/
void EGLwaitVRetrace (void) {
   // just a stub, for now
}

/****************************************************************************\
* Function : EGLgetPixelFormat                                               *
* Desc     : Gets pixel format of the screen (R,G,B masks)                   *
* Input    : None                                                            *
* Output   : Masks                                                           *
\****************************************************************************/
int EGLgetPixelFormat (ulong *RMask, ulong *GMask, ulong *BMask) {


   *RMask = SDLscreen->format->Rmask;
   *GMask = SDLscreen->format->Gmask;
   *BMask = SDLscreen->format->Bmask;

   return EGLok;
}
