/*
 *	Electron Soccer Music Layer - FMOD implementation
 *  Coded by yole, 30.11.2001
 */

#include "el_socc.h"
#include "el_music.h"
#include "els.h"
#include "fxres.h"

#include <stdlib.h>
#include <fmod.h>
#include <fmod_errors.h>

static FMUSIC_MODULE *module = NULL;
static FSOUND_SAMPLE *samples [SAMPLE_LAST];
static HRESELEM sampleIDs [SAMPLE_LAST] = { ID_HIT, ID_HIT1, ID_SELECT };

void ElInitMusic()
{
	for (int i=0; i<SAMPLE_LAST; i++)
		samples [i] = NULL;
}

void ElLoadMusic()
{
	if (NOSOUND == NO)
	{
		if (!FSOUND_Init (44100, 32, FSOUND_INIT_GLOBALFOCUS))
			Error (ErrMusax);

		int ResSize = ResGetSize (ID_MUSAX);
		uchar *ResBuf = ResLoad (ID_MUSAX);
		module = FMUSIC_LoadSongMemory (ResBuf, ResSize);
		if (!module)
			Error (ErrMusax);
		ResUnload (ID_MUSAX);
		FMUSIC_PlaySong (module);
	}
}

void ElShutdownMusic()
{
   if (NOSOUND==NO) {
	   if (module)
		   FMUSIC_FreeSong (module);
	   FSOUND_Close();
   }
}

static FSOUND_SAMPLE *LoadSample (HRESELEM resID, int rate)
{
	int ResSize = ResGetSize (resID);
	char *ResBuf = (char *) ResLoad (resID);
	
	FSOUND_SAMPLE *sample = FSOUND_Sample_Load (FSOUND_FREE, ResBuf,
		FSOUND_LOOP_OFF | FSOUND_8BITS | FSOUND_UNSIGNED | FSOUND_MONO | FSOUND_LOADMEMORY | FSOUND_LOADRAW,
		ResSize);
	if (!sample)
		Error (ErrMusax);
	
	FSOUND_Sample_SetDefaults (sample, -1, 128, -1, -1);
	
	return sample;
}

void ElPlaySample (ElSampleIndex sampleIndex, int rate)
{
	if (samples [sampleIndex] == NULL)
		samples [sampleIndex] = LoadSample (sampleIDs [sampleIndex], rate);

	int channel = FSOUND_PlaySound (FSOUND_FREE, samples [sampleIndex]);
  FSOUND_SetFrequency (channel, rate);
}
