/* $Header: /C/FaerieX/fxFFI.h 1     20.07.98 22:32 Yole $
   FaerieX: .FFI file format declarations
   Written by Yoletir / FaerieFire AKA Dmitry Jemerov, 2:5030/654
*/

#pragma once
#pragma pack(push,1)

/////////////////////////////////////////////////////////////////////////////
// .FFI file format declarations
//
// The .FFI (FaerieFire Image) file format is the format to be used in all
// [FaeF] projects. It has the following features:
//  * Ability to store several images in a single file
//  * 8 (with optional palette), 15, 16 or 24 bpp
//  * Support for transparent images (color key)
//  * Support for compression (not yet implemented)
//
// The general structure of the file is following:
//  * 32 bytes            - the file header (FFI_FileHeader structure)
//  * 768 bytes [if wBPP = 8 && dwFlags & FFIF_PALETTE)
//                        - 8 bit palette for all the images
//  * 4*wImageCount bytes - image offset map
// then (repeated wImageCount times):
//  * 64 bytes            - image header
//  * xxx bytes           - raw or compressed image data

struct FFI_FileHeader
{
  char Sign [4];                       // must be equal to FaeI
  unsigned short wImageCount;          // number of images in the file
  unsigned short wBPP;                 // bits per pixel
  unsigned long dwFlags;               // set of FFIF_xxx constants
  char reserved [20];
}; // 32 bytes

// Bit masks for FFI_FileHeader.dwFlags

#define FFIF_PALETTE   1               // The file contains a palette

struct FFI_ImageHeader
{
  char sName [16];                     // name of the image (optional, may be
                                       // filled with zeros)
  unsigned short wXSize;               // X size of the image
  unsigned short wYSize;               // Y size of the image
  unsigned long dwPackedSize;          // If image is not compressed, equal to
                                       // (wXSize * wYSize * wBPP) / 8,
                                       // otherwise - compressed size of the image
  unsigned long dwColorKey;            // color key of the image
  unsigned long dwFlags;               // set of FFII_xxx constants
  unsigned long dwRMask;               // red bit mask
  unsigned long dwGMask;               // green bit mask
  unsigned long dwBMask;               // blue bit mask
  char reserved [20];
}; // 64 bytes

// Bit masks for FFI_ImageHeader.dwFlags

#define FFII_COLORKEY  1               // The image has a valid dwColorKey
#define FFII_VALIDMASK 2               // dw?Mask fields are valid

#pragma pack(pop)
